@@ -1,3 +1,17 @@
+/**
+ * Licensed under the Apache License, Version 2.0 (the "License"); you may
+ * not use this file except in compliance with the License. You may obtain
+ * a copy of the License at
+ *
+ *    http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+ * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+ * License for the specific language governing permissions and limitations
+ * under the License.
+ */
+
 /* Namespace for core functionality related to modal dialogs.
  *
  * Modals in Horizon are treated as a "stack", e.g new ones are added to the
@@ -29,13 +43,17 @@ horizon.modals.initModal = function (modal) {
 };
 
 /* Creates a modal dialog from the client-side template. */
-horizon.modals.create = function (title, body, confirm, cancel) {
+horizon.modals.create = function (title, body, confirm, cancel, confirmCssClass) {
   if (!cancel) {
     cancel = gettext("Cancel");
   }
   var template = horizon.templates.compiled_templates["#modal_template"],
     params = {
-      title: title, body: body, confirm: confirm, cancel: cancel,
+      title: title,
+      body: body,
+      confirm: confirm,
+      cancel: cancel,
+      confirmCssClass: confirmCssClass || "btn-primary",
       modal_backdrop: horizon.modals.MODAL_BACKDROP
     };
   return $(template.render(params)).appendTo("#modal_wrapper");
@@ -53,13 +71,15 @@ horizon.modals.success = function (data) {
   return modal;
 };
 
-horizon.modals.modal_spinner = function (text) {
+horizon.modals.modal_spinner = function (text, $container) {
+  if (!$container) {
+    $container = $('#modal_wrapper');
+  }
+
   // Adds a spinner with the desired text in a modal window.
-  var template = horizon.templates.compiled_templates["#spinner-modal"];
-  horizon.modals.spinner = $(template.render({text: text}));
-  horizon.modals.spinner.appendTo("#modal_wrapper");
+  horizon.modals.spinner = horizon.loader.modal(text);
+  horizon.modals.spinner.appendTo($container);
   horizon.modals.spinner.modal({backdrop: 'static'});
-  horizon.modals.spinner.find(".modal-body").spin(horizon.conf.spinner_options.modal);
 };
 
 horizon.modals.progress_bar = function (text) {
@@ -89,11 +109,15 @@ horizon.modals.init_wizard = function () {
     }
 
     // Clear old errors.
-    $form.find('div.row div.alert-danger').remove();
+    if ($form.find('div.row div.alert-danger').siblings().length == 0) {
+      $form.find('div.row div.alert-danger').parent().remove();
+    } else {
+      $form.find('div.row div.alert-danger').remove();
+    }
     $form.find('.form-group.has-error').each(function () {
       var $group = $(this);
       $group.removeClass('has-error');
-      $group.find('span.help-block.alert').remove();
+      $group.find('span.help-block').remove();
     });
 
     // Temporarilly remove "disabled" attribute to get the values serialized
@@ -129,8 +153,8 @@ horizon.modals.init_wizard = function () {
             // Add global errors.
             $.each(errors, function (index, error) {
               $fieldset.find('div.row').prepend(
-                '<div class="alert alert-danger">' +
-                error + '</div>');
+                '<div class="col-sm-12"><div class="alert alert-danger">' +
+                error + '</div></div>');
             });
             $fieldset.find('input,  select, textarea').first().focus();
             return;
@@ -139,7 +163,7 @@ horizon.modals.init_wizard = function () {
           $field = $fieldset.find('[name="' + field + '"]');
           $field.closest('.form-group').addClass('has-error');
           $.each(errors, function (index, error) {
-            $field.after(
+            $field.closest(".form-control").after(
               '<span class="help-block">' +
               error + '</span>');
           });
@@ -300,7 +324,26 @@ horizon.addInitFunction(horizon.modals.init = function() {
         $('.ajax-modal, .dropdown-toggle').removeAttr("disabled");
       }
 
-      if (redirect_header) {
+      content_disposition = jqXHR.getResponseHeader("content-disposition");
+      if (content_disposition && content_disposition.lastIndexOf("attachment;") !== -1) {
+        // if the response was a file, we download the data
+        var file_name_regex = /(?:^|\s)filename=(.*?)(?:\s|$)/g;
+        var filename = file_name_regex.exec(content_disposition)[1];
+        var modal_attachment_link = document.createElement('a');
+        modal_attachment_link.setAttribute("id", "modal_attachment_link");
+        modal_attachment_link.setAttribute("style", "display:none");
+        var dataStr = "data:text/plain;charset=utf-8," + encodeURIComponent(data);
+        modal_attachment_link.setAttribute("href", dataStr);
+        modal_attachment_link.setAttribute("download", filename);
+        var modal_wrapper = document.getElementById("modal_wrapper");
+        modal_wrapper.appendChild(modal_attachment_link);
+        modal_attachment_link.click();
+        modal_attachment_link.remove();
+
+        if (redirect_header) {
+          location.href = redirect_header;
+        }
+      } else if (redirect_header) {
         location.href = redirect_header;
       } else if (add_to_field_header) {
         json_data = $.parseJSON(data);
@@ -320,7 +363,7 @@ horizon.addInitFunction(horizon.modals.init = function() {
       } else {
         $('.ajax-modal, .dropdown-toggle').removeAttr("disabled");
         $formElement.closest(".modal").modal("hide");
-        horizon.alert("danger", gettext("There was an error submitting the form. Please try again."));
+        horizon.toast.add("danger", gettext("There was an error submitting the form. Please try again."));
       }
     }
 
@@ -425,7 +468,7 @@ horizon.addInitFunction(horizon.modals.init = function() {
         else {
           if (!horizon.ajax.get_messages(jqXHR)) {
             // Generic error handler. Really generic.
-            horizon.alert("danger", gettext("An error occurred. Please try again later."));
+            horizon.toast.add("danger", gettext("An error occurred. Please try again later."));
           }
         }
       },
