@@ -22,7 +22,7 @@ import logging
 from urllib import parse
 
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from keystoneauth1 import session
 from keystoneauth1 import token_endpoint
@@ -74,7 +74,7 @@ class Service(base.APIDictWrapper):
     _attrs = ['id', 'type', 'name']
 
     def __init__(self, service, region, *args, **kwargs):
-        super(Service, self).__init__(service, *args, **kwargs)
+        super().__init__(service, *args, **kwargs)
         self.public_url = base.get_url_for_service(service, region,
                                                    'publicURL')
         self.url = base.get_url_for_service(service, region,
@@ -90,8 +90,7 @@ class Service(base.APIDictWrapper):
         if(self.type == "identity"):
             return _("%(type)s (%(backend)s backend)") \
                 % {"type": self.type, "backend": keystone_backend_name()}
-        else:
-            return self.type
+        return self.type
 
     def __repr__(self):
         return "<Service: %s>" % self
@@ -158,12 +157,9 @@ def keystoneclient(request, admin=False):
             if domain_token:
                 token_id = getattr(domain_token, 'auth_token', None)
 
-    if admin:
-        if not policy.check((("identity", "admin_required"),), request):
-            raise exceptions.NotAuthorized
-        endpoint_type = 'adminURL'
-    else:
-        endpoint_type = settings.OPENSTACK_ENDPOINT_TYPE
+    if admin and not policy.check((("identity", "admin_required"),), request):
+        raise exceptions.NotAuthorized
+    endpoint_type = settings.OPENSTACK_ENDPOINT_TYPE
 
     # Take care of client connection caching/fetching a new client.
     # Admin vs. non-admin clients are cached separately for token matching.
@@ -179,7 +175,7 @@ def keystoneclient(request, admin=False):
         cacert = settings.OPENSTACK_SSL_CACERT
         verify = verify and cacert
         LOG.debug("Creating a new keystoneclient connection to %s.", endpoint)
-        remote_addr = request.environ.get('REMOTE_ADDR', '')
+        remote_addr = auth_utils.get_client_ip(request)
         token_auth = token_endpoint.Token(endpoint=endpoint,
                                           token=token_id)
         keystone_session = session.Session(auth=token_auth,
@@ -782,7 +778,7 @@ def get_default_role(request):
     to request. Supports lookup by name or id.
     """
     global DEFAULT_ROLE
-    default = settings.OPENSTACK_KEYSTONE_DEFAULT_ROLE
+    default = settings.OPENSTACK_KEYSTONE_DEFAULT_ROLE.lower()
     if default and DEFAULT_ROLE is None:
         try:
             roles = keystoneclient(request, admin=True).roles.list()
@@ -790,7 +786,7 @@ def get_default_role(request):
             roles = []
             exceptions.handle(request)
         for role in roles:
-            if default in (role.id, role.name):
+            if default in (role.id.lower(), role.name.lower()):
                 DEFAULT_ROLE = role
                 break
     return DEFAULT_ROLE
