@@ -16,15 +16,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-
-from django.conf.urls import patterns
-from django.conf.urls import url
+from django.urls import re_path
 
 from openstack_dashboard.dashboards.project.overview import views
 
 
-urlpatterns = patterns(
-    'openstack_dashboard.dashboards.project.overview.views',
-    url(r'^$', views.ProjectOverview.as_view(), name='index'),
-    url(r'^warning$', views.WarningView.as_view(), name='warning'),
-)
+urlpatterns = [
+    re_path(r'^$', views.ProjectOverview.as_view(), name='index'),
+    re_path(r'^warning$', views.WarningView.as_view(), name='warning'),
+]
