@@ -16,20 +16,20 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from django.conf.urls import url
+from django.urls import re_path
 
 from openstack_dashboard.dashboards.project.security_groups import views
 
 urlpatterns = [
-    url(r'^$', views.IndexView.as_view(), name='index'),
-    url(r'^create/$', views.CreateView.as_view(), name='create'),
-    url(r'^(?P<security_group_id>[^/]+)/$',
-        views.DetailView.as_view(),
-        name='detail'),
-    url(r'^(?P<security_group_id>[^/]+)/add_rule/$',
-        views.AddRuleView.as_view(),
-        name='add_rule'),
-    url(r'^(?P<security_group_id>[^/]+)/update/$',
-        views.UpdateView.as_view(),
-        name='update')
+    re_path(r'^$', views.IndexView.as_view(), name='index'),
+    re_path(r'^create/$', views.CreateView.as_view(), name='create'),
+    re_path(r'^(?P<security_group_id>[^/]+)/$',
+            views.DetailView.as_view(),
+            name='detail'),
+    re_path(r'^(?P<security_group_id>[^/]+)/add_rule/$',
+            views.AddRuleView.as_view(),
+            name='add_rule'),
+    re_path(r'^(?P<security_group_id>[^/]+)/update/$',
+            views.UpdateView.as_view(),
+            name='update')
 ]
