@@ -37,7 +37,6 @@
     'horizon.app.core.openstack-service-api.glance',
     'horizon.app.core.openstack-service-api.neutron',
     'horizon.app.core.openstack-service-api.nova',
-    'horizon.app.core.openstack-service-api.novaExtensions',
     'horizon.app.core.openstack-service-api.security-group',
     'horizon.app.core.openstack-service-api.serviceCatalog',
     'horizon.app.core.openstack-service-api.settings',
@@ -57,7 +56,6 @@
    * @param {Object} glanceAPI
    * @param {Object} neutronAPI
    * @param {Object} novaAPI
-   * @param {Object} novaExtensions
    * @param {Object} securityGroup
    * @param {Object} serviceCatalog
    * @param {Object} settings
@@ -81,7 +79,6 @@
     glanceAPI,
     neutronAPI,
     novaAPI,
-    novaExtensions,
     securityGroup,
     serviceCatalog,
     settings,
@@ -132,6 +129,7 @@
        * cloud service properties, they should be READ-ONLY to all UI controllers
        */
 
+      default_availability_zone: 'Any',
       availabilityZones: [],
       flavors: [],
       allowedBootSources: [],
@@ -184,6 +182,7 @@
         description: null,
         // REQUIRED Server Key.  Null allowed.
         user_data: '',
+        default_user_data: '',
         disk_config: 'AUTO',
         // REQUIRED
         flavor: null,
@@ -251,8 +250,13 @@
           function (response) {
             model.defaultBootSource = response;
           });
+        settings.getSetting("OPENSTACK_SERVER_DEFAULT_USER_DATA").then(
+          function (response) {
+            model.newInstanceSpec.default_user_data = response.OPENSTACK_SERVER_DEFAULT_USER_DATA;
+          });
 
         promise = $q.all([
+          launchInstanceDefaults.then(setDefaultValues, noop),
           novaAPI.getAvailabilityZones().then(onGetAvailabilityZones)
             .finally(onGetAvailabilityZonesComplete),
           novaAPI.getFlavors({
@@ -264,7 +268,6 @@
           securityGroup.query().then(onGetSecurityGroups, noop),
           serviceCatalog.ifTypeEnabled('network').then(getNetworks, noop),
           launchInstanceDefaults.then(addImageSourcesIfEnabled, noop),
-          launchInstanceDefaults.then(setDefaultValues, noop),
           launchInstanceDefaults.then(addVolumeSourcesIfEnabled, noop)
         ]);
 
@@ -303,6 +306,9 @@
       if ('hide_create_volume' in defaults) {
         model.newInstanceSpec.hide_create_volume = defaults.hide_create_volume;
       }
+      if ('default_availability_zone' in defaults) {
+        model.default_availability_zone = defaults.default_availability_zone;
+      }
     }
 
     /**
@@ -368,13 +374,33 @@
       if (model.availabilityZones.length === 1) {
         model.newInstanceSpec.availability_zone = model.availabilityZones[0].value;
       } else if (model.availabilityZones.length > 1) {
-        // There are 2 or more; allow ability for nova scheduler to pick,
-        // and make that the default.
+        // There are 2 or more; allow ability for nova scheduler to pick any AZ
         model.availabilityZones.unshift({
           label: gettext("Any Availability Zone"),
           value: ""
         });
-        model.newInstanceSpec.availability_zone = model.availabilityZones[0].value;
+        // if default_availability_zone is Any, pick the first one in the list
+        if (model.default_availability_zone === "Any") {
+          model.newInstanceSpec.availability_zone = model.availabilityZones[0].value;
+        } else {
+          var defaultZone = null;
+          for (var i = 0; i < model.availabilityZones.length; i++) {
+            if (model.availabilityZones[i].value === model.default_availability_zone) {
+              defaultZone = model.availabilityZones[i];
+              break;
+            }
+          }
+          if (defaultZone !== null) {
+            // if default_availability_zone is set, use that AZ by default.
+            model.newInstanceSpec.availability_zone = model.default_availability_zone;
+            // Add "(default)" suffix to the default AZ.
+            defaultZone.label = interpolate(gettext("%s (default)"), [defaultZone.value]);
+          } else {
+            // If the configured default AZ is not included in the AZ list (perhaps
+            // misconfiguration?), use the first one ("Any Availability Zone") by default.
+            model.newInstanceSpec.availability_zone = model.availabilityZones[0].value;
+          }
+        }
       }
 
     }
@@ -567,7 +593,12 @@
 
       if (enabledImage || enabledSnapshot) {
         var filter = {status: 'active', sort_key: 'name', sort_dir: 'asc'};
-        glanceAPI.getImages(filter).then(
+        var filterCommunity = angular.merge({}, filter, {visibility: 'community'});
+        var imagePromises = [
+          glanceAPI.getImages(filter),
+          glanceAPI.getImages(filterCommunity)
+        ];
+        $q.all(imagePromises).then(
           function(data) {
             onGetImageSources(data, enabledImage, enabledSnapshot);
           }
@@ -580,18 +611,11 @@
       var volumeSnapshotDeferred = $q.defer();
       var absoluteLimitsDeferred = $q.defer();
       serviceCatalog
-        .ifTypeEnabled('volumev2')
-        .then(onVolumeServiceEnabled, onCheckVolumeV3);
-      function onCheckVolumeV3() {
-        serviceCatalog
-          .ifTypeEnabled('volumev3')
-          .then(onVolumeServiceEnabled, resolvePromises);
-      }
+        .ifTypeEnabled('volumev3')
+        .then(onVolumeServiceEnabled, resolvePromises);
       function onVolumeServiceEnabled() {
         model.volumeBootable = true;
-        novaExtensions
-          .ifNameEnabled('BlockDeviceMappingV2Boot')
-          .then(onBootToVolumeSupported);
+        model.allowCreateVolumeFromImage = true;
         if (!config || !config.disable_volume) {
           getVolumes().then(resolveVolumes, failVolumes);
           getAbsoluteLimits().then(resolveAbsoluteLimitsDeferred, resolveAbsoluteLimitsDeferred);
@@ -605,9 +629,6 @@
           resolveVolumeSnapshots();
         }
       }
-      function onBootToVolumeSupported() {
-        model.allowCreateVolumeFromImage = true;
-      }
       function getVolumes() {
         return cinderAPI.getVolumes({status: 'available', bootable: 1})
           .then(onGetVolumes);
@@ -678,12 +699,21 @@
       model.imageSnapshots.length = 0;
       model.images.length = 0;
 
-      angular.forEach(data.data.items, function(image) {
-        if (isValidSnapshot(image) && enabledSnapshot) {
-          model.imageSnapshots.push(image);
-        } else if (isValidImage(image) && enabledImage) {
-          model.images.push(image);
-        }
+      var imageIdsProcessed = [];
+
+      angular.forEach(data, function(data) {
+        angular.forEach(data.data.items, function(image) {
+          if (imageIdsProcessed.includes(image.id)) {
+            return;
+          }
+          imageIdsProcessed.push(image.id);
+          if (isValidSnapshot(image) && enabledSnapshot) {
+            model.imageSnapshots.push(image);
+          } else if (isValidImage(image) && enabledImage) {
+            image.name_or_id = image.name || image.id;
+            model.images.push(image);
+          }
+        });
       });
 
       if (enabledImage) {
