@@ -22,7 +22,6 @@
       var model, scope, settings, $q, glance, IMAGE, VOLUME, VOLUME_SNAPSHOT, INSTANCE_SNAPSHOT;
       var cinderEnabled = false;
       var neutronEnabled = false;
-      var novaExtensionsEnabled = false;
       var ifAllowedResolve = true;
 
       var novaApi = {
@@ -131,19 +130,35 @@
 
       beforeEach(module(function($provide) {
         $provide.value('horizon.app.core.openstack-service-api.glance', {
-          getImages: function () {
-            var images = [
-              {container_format: 'aki', properties: {}},
-              {container_format: 'ari', properties: {}},
-              {container_format: 'ami', properties: {}},
-              {container_format: 'raw', properties: {}},
-              {container_format: 'ami', properties: {image_type: 'image'}},
-              {container_format: 'raw', properties: {image_type: 'image'}},
-              {container_format: 'ami', properties: {
-                block_device_mapping: '[{"source_type": "snapshot"}]'}},
-              {container_format: 'raw', properties: {
-                block_device_mapping: '[{"source_type": "snapshot"}]'}}
-            ];
+          getImages: function (params) {
+            var images;
+            if (params.visibility === 'community') {
+              images = [
+                {id: '10', container_format: 'raw', properties: {image_type: 'image'},
+                 visibility: 'community'},
+                // ID 6 is already returned by the first call (below), so this should be ignored.
+                // To clarify the difference, the content here is different intentionally.
+                {id: '6', container_format: 'raw', properties: {image_type: 'image'},
+                 visibility: 'community'},
+              ];
+            } else {
+              images = [
+                // container_format aki and ari are not considered as bootable
+                {id: '1', container_format: 'aki', properties: {}},
+                {id: '2', container_format: 'ari', properties: {}},
+                // The following images are considered as "image" sources.
+                {id: '3', container_format: 'ami', properties: {}, name: 'ami_image'},
+                {id: '4', container_format: 'raw', properties: {}, name: 'raw_image'},
+                {id: '5', container_format: 'ami', properties: {image_type: 'image'}},
+                {id: '6', container_format: 'raw', properties: {image_type: 'image'}},
+                // The following images are considered as "snapshot" sources.
+                {id: '7', container_format: 'ami',
+                 properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}},
+                {id: '8', container_format: 'raw',
+                 properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}},
+                {id: '9', container_format: 'raw', properties: {image_type: 'snapshot'}},
+              ];
+            }
 
             var deferred = $q.defer();
             deferred.resolve({data: {items: images}});
@@ -160,24 +175,25 @@
           }
         });
 
-        beforeEach(function () {
-          settings = {
-            LAUNCH_INSTANCE_DEFAULTS: {
-              create_volume: true,
-              hide_create_volume: false,
-              config_drive: false,
-              disable_image: false,
-              disable_instance_snapshot: false,
-              disable_volume: false,
-              disable_volume_snapshot: false
-            },
-            DEFAULT_BOOT_SOURCE: 'image'
-          };
-          IMAGE = {type: 'image', label: 'Image', selected: true};
-          VOLUME = {type: 'volume', label: 'Volume', selected: false};
-          VOLUME_SNAPSHOT = {type: 'volume_snapshot', label: 'Volume Snapshot', selected: false};
-          INSTANCE_SNAPSHOT = {type: 'snapshot', label: 'Instance Snapshot', selected: false};
-        });
+        settings = {
+          LAUNCH_INSTANCE_DEFAULTS: {
+            create_volume: true,
+            hide_create_volume: false,
+            config_drive: false,
+            disable_image: false,
+            disable_instance_snapshot: false,
+            disable_volume: false,
+            disable_volume_snapshot: false,
+            default_availability_zone: 'Any'
+          },
+          DEFAULT_BOOT_SOURCE: 'image',
+          OPENSTACK_SERVER_DEFAULT_USER_DATA: ''
+        };
+        IMAGE = {type: 'image', label: 'Image', selected: true};
+        VOLUME = {type: 'volume', label: 'Volume', selected: false};
+        VOLUME_SNAPSHOT = {type: 'volume_snapshot', label: 'Volume Snapshot', selected: false};
+        INSTANCE_SNAPSHOT = {type: 'snapshot', label: 'Instance Snapshot', selected: false};
+
         $provide.value('horizon.app.core.openstack-service-api.nova', novaApi);
 
         $provide.value('horizon.app.core.openstack-service-api.security-group', securityGroupApi);
@@ -220,8 +236,6 @@
 
             if (theType === 'network' && neutronEnabled) {
               deferred.resolve();
-            } else if (theType === 'volumev2' && cinderEnabled) {
-              deferred.resolve();
             } else if (theType === 'volumev3' && cinderEnabled) {
               deferred.resolve();
             } else {
@@ -252,20 +266,6 @@
           }
         });
 
-        $provide.value('horizon.app.core.openstack-service-api.novaExtensions', {
-          ifNameEnabled: function() {
-            var deferred = $q.defer();
-
-            if (novaExtensionsEnabled) {
-              deferred.resolve();
-            } else {
-              deferred.reject();
-            }
-
-            return deferred.promise;
-          }
-        });
-
         $provide.value('horizon.app.core.openstack-service-api.settings', {
           getSetting: function(setting) {
             var deferred = $q.defer();
@@ -387,22 +387,23 @@
           expect(model.newInstanceSpec).toBeDefined();
 
           var expectedImages = [
-            {container_format: 'ami', properties: {}},
-            {container_format: 'raw', properties: {}},
-            {container_format: 'ami', properties: {image_type: 'image'}},
-            {container_format: 'raw', properties: {image_type: 'image'}}
+            {id: '3', container_format: 'ami', properties: {}, name: 'ami_image',
+             name_or_id: 'ami_image'},
+            {id: '4', container_format: 'raw', properties: {}, name: 'raw_image',
+             name_or_id: 'raw_image'},
+            {id: '5', container_format: 'ami', properties: {image_type: 'image'}, name_or_id: '5'},
+            {id: '6', container_format: 'raw', properties: {image_type: 'image'}, name_or_id: '6'},
+            {id: '10', container_format: 'raw', properties: {image_type: 'image'}, name_or_id: '10',
+             visibility: 'community'},
           ];
           expect(model.images).toEqual(expectedImages);
 
           var expectedSnapshots = [
-            {
-              container_format: 'ami',
-              properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}
-            },
-            {
-              container_format: 'raw',
-              properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}
-            }
+            {id: '7', container_format: 'ami',
+             properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}},
+            {id: '8', container_format: 'raw',
+             properties: {block_device_mapping: '[{"source_type": "snapshot"}]'}},
+            {id: '9', container_format: 'raw', properties: {image_type: 'snapshot'}},
           ];
           expect(model.imageSnapshots).toEqual(expectedSnapshots);
 
@@ -465,15 +466,6 @@
           expect(model.volumeSnapshots.length).toBe(2);
         });
 
-        it('should disable create volume from image if nova extensions disabled', function() {
-          cinderEnabled = true;
-          novaExtensionsEnabled = false;
-          model.initialize(true);
-          scope.$apply();
-
-          expect(model.allowCreateVolumeFromImage).toBe(false);
-        });
-
         it('should default config_drive to false', function() {
           model.initialize(true);
           scope.$apply();
@@ -513,6 +505,37 @@
           expect(model.newInstanceSpec.create_volume_default).toBe(false);
         });
 
+        it('should default availability_zone to empty based on default', function() {
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toBe('');
+        });
+
+        it('should default availability_zone to empty based on setting', function() {
+          settings.LAUNCH_INSTANCE_DEFAULTS.default_availability_zone = 'Any';
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toBe('');
+        });
+
+        it('should default availability_zone based on setting', function() {
+          settings.LAUNCH_INSTANCE_DEFAULTS.default_availability_zone = 'zone-1';
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toEqual('zone-1');
+        });
+
+        it('should default AZ to empty if AZ configured is not found', function() {
+          settings.LAUNCH_INSTANCE_DEFAULTS.default_availability_zone = 'non-existing';
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toEqual('');
+        });
+
         it('should default hide_create_volume to false if setting not provided', function() {
           delete settings.LAUNCH_INSTANCE_DEFAULTS.hide_create_volume;
           model.initialize(true);
@@ -553,12 +576,13 @@
           expect(glance.getNamespaces.calls.count()).toBe(5);
         });
 
-        it('should not request scheduler hints if scheduler hints disabled', function() {
-          settings.LAUNCH_INSTANCE_DEFAULTS.enable_scheduler_hints = false;
-          model.initialize(true);
-          scope.$apply();
-          expect(glance.getNamespaces.calls.count()).toBe(4);
-        });
+////        Rejections need to be tested differently with AngularJS 1.8.2.
+//        it('should not request scheduler hints if scheduler hints disabled', function() {
+//          settings.LAUNCH_INSTANCE_DEFAULTS.enable_scheduler_hints = false;
+//          model.initialize(true);
+//          scope.$apply();
+//          expect(glance.getNamespaces.calls.count()).toBe(4);
+//        });
 
         it('should set a keypair by default if only one keypair is available', function () {
           var keypair = { keypair: { name: 'key-1' } };
@@ -636,6 +660,22 @@
           expect(model.allowedBootSources).toContain(INSTANCE_SNAPSHOT);
         });
 
+        it('should have proper availability_zone if specific setting is missing', function() {
+          delete settings.LAUNCH_INSTANCE_DEFAULTS.default_availability_zone;
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toBe('');
+        });
+
+        it('should have proper availability_zone if settings are missing', function() {
+          delete settings.LAUNCH_INSTANCE_DEFAULTS;
+          model.initialize(true);
+          scope.$apply();
+
+          expect(model.newInstanceSpec.availability_zone).toBe('');
+        });
+
         it('should have proper allowedBootSources if settings are missing', function() {
           delete settings.LAUNCH_INSTANCE_DEFAULTS;
           model.initialize(true);
@@ -818,12 +858,13 @@
           expect(model.cinderLimits.maxTotalVolumeGigabytes).toBe(1000);
         });
 
-        it('should not fetch server groups if the policy does not allow it', function () {
-          ifAllowedResolve = false;
-          model.initialize(true);
-          scope.$apply();
-          expect(novaApi.getServerGroups.calls.count()).toBe(0);
-        });
+////        Rejections need to be tested differently with AngularJS 1.8.2.
+//        it('should not fetch server groups if the policy does not allow it', function () {
+//          ifAllowedResolve = false;
+//          model.initialize(true);
+//          scope.$apply();
+//          expect(novaApi.getServerGroups.calls.count()).toBe(0);
+//        });
 
         it('should fetch server groups if the policy allows it', function () {
           ifAllowedResolve = true;
@@ -843,7 +884,7 @@
         // This is here to ensure that as people add/change items, they
         // don't forget to implement tests for them.
         it('has the right number of properties', function() {
-          expect(Object.keys(model.newInstanceSpec).length).toBe(22);
+          expect(Object.keys(model.newInstanceSpec).length).toBe(23);
         });
 
         it('sets availability zone to null', function() {
@@ -870,6 +911,10 @@
           expect(model.newInstanceSpec.user_data).toBe('');
         });
 
+        it('sets default user data to an empty string', function() {
+          expect(model.newInstanceSpec.default_user_data).toBe('');
+        });
+
         it('sets disk config to AUTO', function() {
           expect(model.newInstanceSpec.disk_config).toBe('AUTO');
         });
@@ -1066,7 +1111,7 @@
           expect(finalSpec.key_pair).toBeUndefined();
         });
 
-        it('stips null properties', function() {
+        it('strips null properties', function() {
           model.newInstanceSpec.useless = null;
 
           var finalSpec = model.createInstance();
