@@ -33,11 +33,6 @@
   LaunchInstanceSourceController.$inject = [
     '$scope',
     'horizon.dashboard.project.workflow.launch-instance.boot-source-types',
-    'bytesFilter',
-    'dateFilter',
-    'decodeFilter',
-    'diskFormatFilter',
-    'gbFilter',
     'horizon.dashboard.project.workflow.launch-instance.basePath',
     'horizon.framework.widgets.transfer-table.events',
     'horizon.framework.widgets.magic-search.events'
@@ -45,17 +40,13 @@
 
   function LaunchInstanceSourceController($scope,
     bootSourceTypes,
-    bytesFilter,
-    dateFilter,
-    decodeFilter,
-    diskFormatFilter,
-    gbFilter,
     basePath,
     events,
     magicSearchEvents
   ) {
 
     var ctrl = this;
+    ctrl.volumeSizeError = gettext('Volume size is required and must be an integer');
 
     // Error text for invalid fields
     /*eslint-disable max-len */
@@ -77,8 +68,6 @@
     /*
      * Transfer table
      */
-    ctrl.tableHeadCells = [];
-    ctrl.tableBodyCells = [];
     ctrl.tableData = {
       available: [],
       allocated: selection,
@@ -86,7 +75,21 @@
       displayedAllocated: []
     };
     ctrl.helpText = {};
-    ctrl.sourceDetails = basePath + 'source/source-details.html';
+
+    ctrl.availableTableConfig = {
+      selectAll: false,
+      trackId: 'id',
+      detailsTemplateUrl: basePath + 'source/source-details.html',
+      columns: []
+    };
+
+    ctrl.allocatedTableConfig = angular.copy(ctrl.availableTableConfig);
+    ctrl.allocatedTableConfig.noItemsMessage = gettext(
+      'Select an item from Available items below');
+
+    ctrl.tableLimits = {
+      maxAllocation: 1
+    };
 
     var bootSources = {
       image: {
@@ -98,19 +101,19 @@
       snapshot: {
         available: $scope.model.imageSnapshots,
         allocated: selection,
-        displayedAvailable: [],
+        displayedAvailable: $scope.model.imageSnapshots,
         displayedAllocated: selection
       },
       volume: {
         available: $scope.model.volumes,
         allocated: selection,
-        displayedAvailable: [],
+        displayedAvailable: $scope.model.volumes,
         displayedAllocated: selection
       },
       volume_snapshot: {
         available: $scope.model.volumeSnapshots,
         allocated: selection,
-        displayedAvailable: [],
+        displayedAvailable: $scope.model.volumeSnapshots,
         displayedAllocated: selection
       }
     };
@@ -129,73 +132,98 @@
       { label: gettext('VMDK'), key: 'vmdk' }
     ];
 
-    // Mapping for dynamic table headers
-    var tableHeadCellsMap = {
+    var diskFormatsObj = diskFormats.reduce(function (acc, cur) {
+      acc[cur.key] = cur.label;
+      return acc;
+    }, {});
+
+    function getImageDiskFormat(key) {
+      return diskFormatsObj[key];
+    }
+
+    function getVolumeDiskFormat(data) {
+      return diskFormatsObj[data.disk_format];
+    }
+
+    var statuses = [
+          { label: gettext('Available'), key: 'available' },
+          { label: gettext('Creating'), key: 'creating' },
+          { label: gettext('Deleting'), key: 'deleting' },
+          { label: gettext('Error'), key: 'error' },
+          { label: gettext('Error Deleting'), key: 'error_deleting' }
+    ];
+
+    var statusesObj = statuses.reduce(function (acc, cur) {
+      acc[cur.key] = cur.label;
+      return acc;
+    }, {});
+
+    function getStatus(status) {
+      return statusesObj[status];
+    }
+
+    // Mapping for dynamic table columns
+    var tableColumnsMap = {
       image: [
-        { text: gettext('Name'), sortable: true, sortDefault: true },
-        { text: gettext('Updated'), sortable: true },
-        { text: gettext('Size'), classList: ['number'], sortable: true },
-        { text: gettext('Type'), sortable: true },
-        { text: gettext('Visibility'), sortable: true }
+        { id: 'name_or_id', title: gettext('Name'), priority: 1 },
+        { id: 'updated_at', title: gettext('Updated'), filters: ['simpleDate'], priority: 2 },
+        { id: 'size', title: gettext('Size'), filters: ['bytes'], priority: 2 },
+        { id: 'disk_format', title: gettext('Format'),
+          filters: [getImageDiskFormat], priority: 2 },
+        { id: 'visibility', title: gettext('Visibility'), filters: [getVisibility], priority: 2 }
       ],
       snapshot: [
-        { text: gettext('Name'), sortable: true, sortDefault: true },
-        { text: gettext('Updated'), sortable: true },
-        { text: gettext('Size'), classList: ['number'], sortable: true },
-        { text: gettext('Type'), sortable: true },
-        { text: gettext('Visibility'), sortable: true }
+        { id: 'name', title: gettext('Name'), priority: 1 },
+        { id: 'updated_at', title: gettext('Updated'), filters: ['simpleDate'], priority: 2 },
+        { id: 'size', title: gettext('Size'), filters: ['bytes'], priority: 2 },
+        { id: 'disk_format', title: gettext('Format'),
+          filters: [getImageDiskFormat], priority: 2 },
+        { id: 'visibility', title: gettext('Visibility'), filters: [getVisibility], priority: 2 }
       ],
       volume: [
-        { text: gettext('Name'), sortable: true, sortDefault: true },
-        { text: gettext('Description'), sortable: true },
-        { text: gettext('Size'), classList: ['number'], sortable: true },
-        { text: gettext('Type'), sortable: true },
-        { text: gettext('Availability Zone'), sortable: true }
+        { id: 'name', title: gettext('Name'), priority: 1 },
+        { id: 'description', title: gettext('Description'), filters: ['noValue'], priority: 2 },
+        { id: 'size', title: gettext('Size'), filters: ['gb'], priority: 2 },
+        { id: 'volume_image_metadata', title: gettext('Type'),
+          filters: [getVolumeDiskFormat], priority: 2 },
+        { id: 'availability_zone', title: gettext('Availability Zone'), priority: 2 }
       ],
       volume_snapshot: [
-        { text: gettext('Name'), sortable: true, sortDefault: true },
-        { text: gettext('Description'), sortable: true },
-        { text: gettext('Size'), classList: ['number'], sortable: true },
-        { text: gettext('Created'), sortable: true },
-        { text: gettext('Status'), sortable: true }
+        { id: 'name', title: gettext('Name'), priority: 1 },
+        { id: 'description', title: gettext('Description'), filters: ['noValue'], priority: 2 },
+        { id: 'size', title: gettext('Size'), filters: ['gb'], priority: 2 },
+        { id: 'created_at', title: gettext('Created'), filters: ['simpleDate'], priority: 2 },
+        { id: 'status', title: gettext('Status'), filters: [getStatus], priority: 2 }
       ]
     };
 
     // Map Visibility data so we can decode true/false to Public/Private
-    var _visibilitymap = { true: gettext('Public'), false: gettext('Private') };
-
-    // Mapping for dynamic table data
-    var tableBodyCellsMap = {
-      image: [
-        { key: 'name', classList: ['hi-light', 'word-break'] },
-        { key: 'updated_at', filter: dateFilter, filterArg: 'short' },
-        { key: 'size', filter: bytesFilter, classList: ['number'] },
-        { key: 'disk_format', filter: diskFormatFilter, filterRawData: true },
-        { key: 'is_public', filter: decodeFilter, filterArg: _visibilitymap }
-      ],
-      snapshot: [
-        { key: 'name', classList: ['hi-light', 'word-break'] },
-        { key: 'updated_at', filter: dateFilter, filterArg: 'short' },
-        { key: 'size', filter: bytesFilter, classList: ['number'] },
-        { key: 'disk_format', filter: diskFormatFilter, filterRawData: true },
-        { key: 'is_public', filter: decodeFilter, filterArg: _visibilitymap }
-      ],
-      volume: [
-        { key: 'name', classList: ['hi-light', 'word-break'] },
-        { key: 'description' },
-        { key: 'size', filter: gbFilter, classList: ['number'] },
-        { key: 'volume_image_metadata', filter: diskFormatFilter },
-        { key: 'availability_zone' }
-      ],
-      volume_snapshot: [
-        { key: 'name', classList: ['hi-light', 'word-break'] },
-        { key: 'description' },
-        { key: 'size', filter: gbFilter, classList: ['number'] },
-        { key: 'created_at', filter: dateFilter, filterArg: 'short' },
-        { key: 'status' }
-      ]
+    var _visibilitymap = { 'public': gettext('Public'),
+                           'private': gettext('Private'),
+                           'shared': gettext('Shared'),
+                           'community': gettext('Community')
     };
 
+    function getVisibility(visibility) {
+      return _visibilitymap[visibility];
+    }
+
+    /**
+     * Creates a map of functions that sort by the key at a given index for
+     * the selected object
+     */
+    ctrl.sortByField = [];
+
+    var sortFunction = function(columnIndex, comparedObject) {
+      var cell = tableColumnsMap[ctrl.currentBootSource];
+      var key = cell[columnIndex].id;
+      return comparedObject[key];
+    };
+
+    for (var i = 0; i < 5; ++i) {
+      ctrl.sortByField.push(sortFunction.bind(null, i));
+    }
+
     /**
      * Filtering - client-side MagicSearch
      */
@@ -236,16 +264,10 @@
         label: gettext('Status'),
         name: 'status',
         singleton: true,
-        options: [
-          { label: gettext('Available'), key: 'available' },
-          { label: gettext('Creating'), key: 'creating' },
-          { label: gettext('Deleting'), key: 'deleting' },
-          { label: gettext('Error'), key: 'error' },
-          { label: gettext('Error Deleting'), key: 'error_deleting' }
-        ]
+        options: statuses
       },
       type: {
-        label: gettext('Type'),
+        label: gettext('Format'),
         name: 'disk_format',
         singleton: true,
         options: diskFormats
@@ -257,11 +279,13 @@
       },
       visibility: {
         label: gettext('Visibility'),
-        name: 'is_public',
+        name: 'visibility',
         singleton: true,
         options: [
-          { label: gettext('Public'), key: 'true' },
-          { label: gettext('Private'), key: 'false' }
+          { label: gettext('Public'), key: 'public' },
+          { label: gettext('Private'), key: 'private' },
+          { label: gettext('Shared With Project'), key: 'shared' },
+          { label: gettext('Community'), key: 'community' }
         ]
       },
       volumeType: {
@@ -318,7 +342,7 @@
       },
       function onBootSourceChange(newValue, oldValue) {
         if (newValue !== oldValue) {
-          $scope.$broadcast(events.AVAIL_CHANGED, {
+          $scope.$broadcast(events.TABLES_CHANGED, {
             'data': bootSources[newValue]
           });
         }
@@ -340,6 +364,21 @@
       }
     );
 
+    var imageSnapshotsWatcher = $scope.$watchCollection(
+      function getImageSnapshots() {
+        return $scope.model.imageSnapshots;
+      },
+      function onImageSnapshotsChange() {
+        $scope.initPromise.then(function () {
+          $scope.$applyAsync(function () {
+            if ($scope.launchContext.imageId) {
+              setSourceImageSnapshotWithId($scope.launchContext.imageId);
+            }
+          });
+        });
+      }
+    );
+
     var volumeWatcher = $scope.$watchCollection(
       function getVolumes() {
         return $scope.model.volumes;
@@ -371,21 +410,40 @@
     );
 
     // When the allowedboot list changes, change the source_type
-    // and update the table for the new source selection. Only done
-    // with the first item for the list
+    // and update the table for the new source selection. The devault value is
+    // set by the DEFAULT_BOOT_SOURCE config option.
+    // The boot source is changed only if the selected value is not included
+    // in the updated list (newValue)
     var allowedBootSourcesWatcher = $scope.$watchCollection(
       function getAllowedBootSources() {
         return $scope.model.allowedBootSources;
       },
       function changeBootSource(newValue) {
-        if (angular.isArray(newValue) && newValue.length > 0 &&
-          !$scope.model.newInstanceSpec.source_type) {
-          updateBootSourceSelection(newValue[0].type);
-          $scope.model.newInstanceSpec.source_type = newValue[0];
+        if (angular.isArray(newValue) && newValue.length > 0 ) {
+          var opt = newValue[0];
+          for (var index = 0; index < newValue.length; index++) {
+            if (newValue[index].selected) {
+              opt = newValue[index];
+            }
+          }
+          updateBootSourceSelection(opt.type);
+          $scope.model.newInstanceSpec.source_type = opt;
         }
       }
     );
 
+    var flavorWatcher = $scope.$watchCollection(function () {
+      return $scope.model.newInstanceSpec.flavor;
+    }, function setVolumeSize() {
+      // Set the volume size if a flavor is selected and it requires
+      // more disk space than what the user specified.
+      var newInstanceSpec = $scope.model.newInstanceSpec;
+      var flavor = newInstanceSpec.flavor;
+      if (flavor && (newInstanceSpec.vol_size < flavor.disk)) {
+        newInstanceSpec.vol_size = flavor.disk;
+      }
+    });
+
     // Explicitly remove watchers on destruction of this controller
     $scope.$on('$destroy', function() {
       allowedBootSourcesWatcher();
@@ -393,21 +451,28 @@
       allocatedWatcher();
       bootSourceWatcher();
       imagesWatcher();
+      imageSnapshotsWatcher();
       volumeWatcher();
       snapshotWatcher();
+      flavorWatcher();
     });
 
     ////////////////////
 
-    function updateBootSourceSelection(selectedSource) {
-      ctrl.currentBootSource = selectedSource;
-      if ((selectedSource === bootSourceTypes.IMAGE) && $scope.model.volumeBootable) {
-        $scope.model.newInstanceSpec.vol_create = true;
+    function updateBootSourceSelection(selectedSource, preSelection) {
+      if (ctrl.currentBootSource !== selectedSource) {
+        ctrl.selection.length = 0;
+        ctrl.currentBootSource = selectedSource;
+      }
+      if ((selectedSource === bootSourceTypes.IMAGE ||
+           selectedSource === bootSourceTypes.INSTANCE_SNAPSHOT) && $scope.model.volumeBootable) {
+        $scope.model.newInstanceSpec.vol_create =
+          $scope.model.newInstanceSpec.create_volume_default;
       } else {
         $scope.model.newInstanceSpec.vol_create = false;
       }
       $scope.model.newInstanceSpec.vol_delete_on_instance_delete = false;
-      changeBootSource(selectedSource);
+      changeBootSource(selectedSource, preSelection);
       validateBootSourceType();
     }
 
@@ -415,14 +480,13 @@
     function changeBootSource(key, preSelection) {
       updateDataSource(key, preSelection);
       updateHelpText(key);
-      updateTableHeadCells(key);
-      updateTableBodyCells(key);
+      updateTableColumns(key);
       updateFacets(key);
     }
 
     function updateDataSource(key, preSelection) {
-      selection.length = 0;
       if (preSelection) {
+        ctrl.selection.length = 0;
         push.apply(selection, preSelection);
       }
       angular.extend(ctrl.tableData, bootSources[key]);
@@ -438,12 +502,9 @@
       });
     }
 
-    function updateTableHeadCells(key) {
-      refillArray(ctrl.tableHeadCells, tableHeadCellsMap[key]);
-    }
-
-    function updateTableBodyCells(key) {
-      refillArray(ctrl.tableBodyCells, tableBodyCellsMap[key]);
+    function updateTableColumns(key) {
+      refillArray(ctrl.availableTableConfig.columns, tableColumnsMap[key]);
+      refillArray(ctrl.allocatedTableConfig.columns, tableColumnsMap[key]);
     }
 
     function updateFacets(key) {
@@ -467,8 +528,9 @@
     function checkVolumeForImage() {
       var source = selection[0];
 
-      if (source && ctrl.currentBootSource === bootSourceTypes.IMAGE) {
-        var imageGb = source.size * 1e-9;
+      if (source && ctrl.currentBootSource === bootSourceTypes.IMAGE ||
+          source && ctrl.currentBootSource === bootSourceTypes.INSTANCE_SNAPSHOT ) {
+        var imageGb = source.size / 1073741824.0;
         var imageDisk = source.min_disk;
         ctrl.minVolumeSize = Math.ceil(Math.max(imageGb, imageDisk));
         if ($scope.model.newInstanceSpec.vol_size < ctrl.minVolumeSize) {
@@ -476,10 +538,10 @@
         }
         var volumeSizeText = gettext('The volume size must be at least %(minVolumeSize)s GB');
         var volumeSizeObj = { minVolumeSize: ctrl.minVolumeSize };
-        ctrl.volumeSizeError = interpolate(volumeSizeText, volumeSizeObj, true);
+        ctrl.minVolumeSizeError = interpolate(volumeSizeText, volumeSizeObj, true);
       } else {
         ctrl.minVolumeSize = 0;
-        ctrl.volumeSizeError = gettext('Volume size is required and must be an integer');
+        ctrl.minVolumeSizeError = gettext('Volume size is required and must be an integer');
       }
     }
 
@@ -514,36 +576,44 @@
     function setSourceImageWithId(id) {
       var pre = findSourceById($scope.model.images, id);
       if (pre) {
-        changeBootSource(bootSourceTypes.IMAGE, [pre]);
+        updateBootSourceSelection(bootSourceTypes.IMAGE, [pre]);
         $scope.model.newInstanceSpec.source_type = {
           type: bootSourceTypes.IMAGE,
           label: gettext('Image')
         };
-        ctrl.currentBootSource = bootSourceTypes.IMAGE;
+      }
+    }
+
+    function setSourceImageSnapshotWithId(id) {
+      var pre = findSourceById($scope.model.imageSnapshots, id);
+      if (pre) {
+        updateBootSourceSelection(bootSourceTypes.INSTANCE_SNAPSHOT, [pre]);
+        $scope.model.newInstanceSpec.source_type = {
+          type: bootSourceTypes.INSTANCE_SNAPSHOT,
+          label: gettext('Snapshot')
+        };
       }
     }
 
     function setSourceVolumeWithId(id) {
       var pre = findSourceById($scope.model.volumes, id);
       if (pre) {
-        changeBootSource(bootSourceTypes.VOLUME, [pre]);
+        updateBootSourceSelection(bootSourceTypes.VOLUME, [pre]);
         $scope.model.newInstanceSpec.source_type = {
           type: bootSourceTypes.VOLUME,
           label: gettext('Volume')
         };
-        ctrl.currentBootSource = bootSourceTypes.VOLUME;
       }
     }
 
     function setSourceSnapshotWithId(id) {
       var pre = findSourceById($scope.model.volumeSnapshots, id);
       if (pre) {
-        changeBootSource(bootSourceTypes.VOLUME_SNAPSHOT, [pre]);
+        updateBootSourceSelection(bootSourceTypes.VOLUME_SNAPSHOT, [pre]);
         $scope.model.newInstanceSpec.source_type = {
           type: bootSourceTypes.VOLUME_SNAPSHOT,
           label: gettext('Snapshot')
         };
-        ctrl.currentBootSource = bootSourceTypes.VOLUME_SNAPSHOT;
       }
     }
   }
