@@ -180,10 +180,16 @@ module.exports = function (config) {
       dir: '../cover/openstack_dashboard'
     },
 
+    client: {
+      jasmine: {
+        random: false
+      }
+    },
+
     // Coverage threshold values.
     thresholdReporter: {
       statements: 96, // target 100
-      branches: 93, // target 100
+      branches: 92, // target 100
       functions: 95, // target 100
       lines: 96 // target 100
     }
